#!/bin/ash
# add/remove/change drive-icon to ROX-Filer pinboard
# (C) James Budiono 2012
# License: GNU GPL Version 3 or later
#
# called from udev where ACTION=add/remove/change and SUBSYSTEM=block, executed as root, no X
# uses uses ACTION, DEVNAME, DEVPATH environments

### configurations
export BOOTSTATE_PATH=/etc/BOOTSTATE		# udev calls us only with udev environments
EVENTMANAGER_CONFIG=/etc/eventmanager
DRIVE_ICON_ROOT=/tmp/fatdog-drive-icons 	# $USER will be appended to it
PINBOARD_FILE=/.config/rox.sourceforge.net/ROX-Filer/PuppyPin	# $HOME will be added in front of this
MARGIN=32	# margins in pixels - subtracted from top/down/left/right
ICON_DX=64	# assumed icon size in pixels
ICON_DY=64	# assumed icon size in pixels
GRID_X=16	# check every $GRID pixels
GRID_Y=16	# check every $GRID pixels

### system configuration - no need to change
ICON_IMAGE_ROOT=/usr/share/midi-icons

MUTEX_TIMEOUT=50 					# 5 seconds (unit is 1/10 seconds)
F_MUTEX=/tmp/fatdog-drive-icon-busy # $USER will be appended to this
F_ENABLE=/tmp/fatdog-drive-icon-enable	# $USER will be appended to this

PATH=/usr/X11/bin:/usr/local/bin:$PATH
DEBUG_FILE=/tmp/fatdog-drive-icon.log
DEBUG=


#################### environments #####################

# find the first local X server, construct DISPLAY variable
get_DISPLAY() {
	export DISPLAY=$(ls /tmp/.X*-lock 2> /dev/null | head -n 1 | sed 's/-lock//; s/.*\.//; s/X/:/')
	log_debug "DISPLAY is $DISPLAY"
}

# find the X user, assume this will be the owner of xinit process - construct USER variable
get_USER() {
	export USER=$(ps -eo user,comm | awk '/xinit$/ {print $1}')
	log_debug "USER is $USER"
}

# find the HOME directory of $USER
get_HOME() {
	export HOME=$(awk -F: "/^$USER/ {print \$6}" /etc/passwd)
	log_debug "HOME is $HOME"
}

# setup environments - DISPLAY USER HOME F_MUTEX DRIVE_ICON_ROOT PINBOARD_FILE F_ENABLE
setup_env() {
	get_DISPLAY
	get_USER
	case $DISPLAY in "") exit ;; esac # if there is no display, leave		
	case $USER in "") exit ;; esac # if there is no user, leave	
	get_HOME	
	PINBOARD_FILE=$HOME/$PINBOARD_FILE
	DRIVE_ICON_ROOT=$DRIVE_ICON_ROOT.$USER
	F_MUTEX=$F_MUTEX.$USER
	F_ENABLE=$F_ENABLE.$USER
}

# output $@ to debug file
log_debug() {
	[ "$DEBUG" ] && echo $(basename $0): "$@" >> $DEBUG_FILE
}

#################### utilities #####################

### simple, stupid, but quick way to determine drive type 
# $1-drivename $2-devpath returns DRIVE_TYPE
guess_drive_type() {
	DRIVE_TYPE=any
	case $2 in
		*usb*) DRIVE_TYPE=usbdrv
	esac
	case $1 in 
		fd*) DRIVE_TYPE=floppy ;;
		sr*) DRIVE_TYPE=optical ;;
		mmc*) DRIVE_TYPE=card ;;
	esac
}


#################### mutex management #####################

# get mutex
get_mutex() {
	local timeout=$MUTEX_TIMEOUT
	while ! mkdir $F_MUTEX; do
		sleep 0.1; 
		case $timeout in 
			0) break ;;
			*) timeout=$((timeout - 1)) ;;
		esac
	done
	log_debug lock $DRIVENAME
}

# release mutex
release_mutex() {
	rm -rf $F_MUTEX
	log_debug unlock $DRIVENAME
}

#################### screen management #####################

### get coordinates of a free location for adding icons
# $1-aligment = top, bottom, left, right (default is bottom)
# (top & bottom the same, only starting position is different)
# (left & right the same, only starting position is different)
# returns COORD_X and COORD_Y for historical reasons
get_free_coord() {
	local win_x win_y win_dx win_dy x y dx dy xx yy nextx nexty countx county orientation collision
	orientation=bottom
	[ $1 ] && orientation=$1
	
	# get bounding screen coordinate
	win_x=0 win_y=0
	win_dx=$(xwininfo -root  | sed '/Width:/ !d; s/.*: //')
	win_dy=$(xwininfo -root  | sed '/Height:/ !d; s/.*: //')
	
	# account for panels - supports only jwm and lxpanel
	if pidof jwm > /dev/null; then
		get_panel_box JWM
	elif pidof lxpanel.bin > /dev/null; then
		get_panel_box panel
	fi
	
	# reduce again with margins
	win_x=$((win_x + MARGIN)) win_y=$((win_y + MARGIN)) 
	win_dx=$((win_dx - MARGIN)) win_dy=$((win_dy - MARGIN))
	#echo $win_x $win_y $win_dx $win_dy
	
	# starting position
	case $orientation in
		top) x=$win_x y=$win_y nextx=$GRID_X nexty=0 ;;
		bottom) x=$win_x y=$((win_dy - ICON_DY / 2)) nextx=$GRID_X nexty=0 ;;
		left) x=$win_x y=$win_y nextx=0 nexty=$GRID_Y ;;
		right) x=$((win_dx - ICON_DX / 2)) y=$win_y nextx=0 nexty=$GRID_Y ;;
	esac
	dx=$((x + ICON_DX)) dy=$((y + ICON_DY)) xx=$x yy=$y
	#echo $x $y $dx $dy $nextx $nexty
	
	# load existing icon positions from puppypin
	countx=0 county=0
	while read p; do
		case $p in
			"") true ;; # blank - do nothing
			*)  case ${p%=*} in
					x) countx=$((countx + 1))
					   eval pin_x_$countx=${p#*=} ;;
					y) county=$((county + 1))
					   eval pin_y_$county=${p#*=} ;;				   
				esac
		esac
	done << EOF
$(xml2 < $PINBOARD_FILE | awk -F@ '{print $2}' )
EOF
	
	# check for collisions (overlapping icons)
	while true; do
		collision=
		for p in $(seq 1 $countx); do
			eval pinx=\$pin_x_$p
			eval piny=\$pin_y_$p
			[ \( \( $dx -ge $pinx -a $dx -lt $((pinx + ICON_DX)) \) -o \
			     \( $x  -ge $pinx -a $x  -lt $((pinx + ICON_DX)) \) \) -a \
			  \( \( $dy -ge $piny -a $dy -lt $((piny + ICON_DY)) \) -o \
			     \( $y  -ge $piny -a $y  -lt $((piny + ICON_DY)) \) \) ] && collision=yes && break
		done
		[ -z $collision ] && break
		
		# fail, move to next slot
		x=$((x + nextx)) y=$((y + nexty)) dx=$((dx + nextx)) dy=$((dy + nexty)) 
		
		# make sure we don't go past edge of screen - not foolproof but good enough
		if [ $dx -ge $win_dx ]; then
			x=$xx dx=$((x + ICON_DX)) 
			case $orientation in 
				top)    y=$((y + ICON_DY)) dy=$((dy + ICON_DY)) ;;
				bottom) y=$((y - ICON_DY)) dy=$((dy - ICON_DY)) ;;
			esac
		fi
		
		if [ $dy -ge $win_dy ]; then
			y=$yy dy=$((y + ICON_DY)) 
			case $orientation in 
				left)  x=$((x + ICON_DX)) dx=$((dx + ICON_DX)) ;;
				right) x=$((x - ICON_DX)) dx=$((dx - ICON_DX)) ;;
			esac
		fi
	done
	
	# clear out the temporary array variables
	while read p; do
		unset $p
	done << EOF
$(set | grep pin)
EOF
	COORD_X=$x
	COORD_Y=$y
	log_debug get_free_coord $x $y
}
### helper for get_free_coord
# reduce window size by space taken by panels
# $1-window name
get_panel_box() {
	local x y dx dy
	dx=$(xwininfo -name $1  | sed '/Width:/ !d; s/.*: //')
	dy=$(xwininfo -name $1  | sed '/Height:/ !d; s/.*: //')
	x=$(xwininfo -name $1  | sed '/Absolute upper-left X:/ !d; s/.*: //')
	y=$(xwininfo -name $1  | sed '/Absolute upper-left Y:/ !d; s/.*: //')
	#echo $x $y $dx $dy
	
	if [ $dx = $win_dx ]; then # horizontal orientation
		if [ $y = 0 ]; then # top
			win_y=$dy
		else # bottom
			win_dy=$y
		fi
	else # assume vertical orientation
		if [ $x = 0 ]; then # left
			win_x=$dx
		else # right
			win_dx=$x
		fi	
	fi
}


#################### rox-specific stuff #####################

### add icon to rox
#$1-path $2-COORD_X $3-COORD_Y $4-label $5-args
rox_add_icon() {
	pidof ROX-Filer > /dev/null || return
	log_debug rox_add_icon $@
	su $USER -c "rox -R" << EOF
<?xml version="1.0"?>
<env:Envelope xmlns:env="http://www.w3.org/2001/12/soap-envelope">
	<env:Body xmlns="http://rox.sourceforge.net/SOAP/ROX-Filer">
		<PinboardAdd>
			<Path>$1</Path>
			<X>$2</X>
			<Y>$3</Y>
			<Label>$4</Label>
			<Args>$5</Args>
		</PinboardAdd>
	</env:Body>
</env:Envelope>
EOF
}

### remove icon from rox
#$1-path
rox_remove_icon() {
	pidof ROX-Filer > /dev/null || return	
	log_debug rox_remove_icon $@
	su $USER -c "rox -R" << EOF
<?xml version="1.0"?>
<env:Envelope xmlns:env="http://www.w3.org/2001/12/soap-envelope">
	<env:Body xmlns="http://rox.sourceforge.net/SOAP/ROX-Filer">
		<PinboardRemove>
			<Path>$1</Path>
		</PinboardRemove>
	</env:Body>
</env:Envelope>
EOF
}

# svg icon inspired by technosaurus (Brad Conroy), see http://www.murga-linux.com/puppy/viewtopic.php?t=76431
#$1-path $2-type
create_icon_file() {
	local DRVICON=drive48.png
	case $2 in
		usbdrv) DRVICON=usbdrv48.png ;;
		card) DRVICON=card48.png ;;
		optical) DRVICON=optical48.png ;;		
		floppy) DRVICON=floppy48.png ;;
	esac
	log_debug create_icon_image $DRVICON
	cat > $1/.DirIcon << EOF
<svg width="48" height="48" id="svg1" xmlns:xlink="http://www.w3.org/1999/xlink">
<image xlink:href="$ICON_IMAGE_ROOT/$DRVICON" width="48" height="48"/>
</svg>
EOF
}

### create rox application to handle device
# $1-path, $2-type, $3-devname $4-summary
create_rox_app() {
	{
	log_debug create_rox_app $@
	# standard menus
	cat << EOF
<?xml version="1.0"?>
<AppInfo>
	<Summary>$4</Summary>
	<About>
		<Purpose>Fatdog desktop drive icons</Purpose>
		<Authors>Re-written for Fatdog64-600, James Budiono 2012</Authors>
		<License>GNU GPL Version 3 or later</License>
	</About>
	<AppMenu>
		<Item option="umount $3" icon="gtk-cancel"><Label>Un-mount ${3##*/} if currently mounted</Label></Item>
		<Item option="pmount" icon="gtk-execute"><Label>Puppy Drive Mounter</Label></Item>
EOF
	# extra menus for optical drives
	case $2 in
		optical) cat << EOF
		<Item option="explore $3" icon="gtk-cdrom"><Label>Explore optical disk</Label></Item>
		<Item option="eject $3" icon="gtk-cdrom"><Label>Eject optical disk</Label></Item>		
EOF
		;;
	esac
	
	# closing tags
	echo "</AppMenu></AppInfo>"
	} > $1/AppInfo.xml
	
	create_icon_file $1 $2
	ln -s /usr/sbin/fatdog-drive-icon-action-handler.sh $1/AppRun
}

# $1-FS_TYPE returns SUMMARY
get_summary() {
	#SUMMARY=$(disktype $DEVNAME | sed -n '/Volume name/ { s/.*\(".*"\).*/\1/; p}')
	SUMMARY=$(busybox blkid $DEVNAME | sed -n '/LABEL/ {s/.*LABEL="\([^"]*\)".*/\1/; p}')
	read p < /sys/$DEVPATH/size
	p=$(( p / 2048 )) # convert to MB assuming 512-byte units
	[ $p -ge 2048 ] && SUMMARY="$SUMMARY $(( p / 1024))GB" || SUMMARY="$SUMMARY ${p}MB"
	[ "$1" ] && SUMMARY="$SUMMARY $1"
}

#################### main udev handlers here #####################

### add handler - add icons to rox
udev_add_handler() {
	PROCEED=no FS_TYPE=""
	case $DRIVE_TYPE in
		floppy) PROCEED=yes ;;									# floppy - always display
		optical) cddetect_quick -d $DEVNAME && PROCEED=yes ;;	# optical - show if there is disc inside
		*)	FS_TYPE=$(guess_fstype $DEVNAME)					# others - show icon if we recognise the filesystem
			case "$FS_TYPE" in 	  				
				unknown|""|Usage*) log_debug "$DEVNAME filesystem unknown" ;; # unknown - do nothing
				*) PROCEED=yes;;
			esac ;;
	esac

	case $PROCEED in 
		yes) if [ ! -d $DRIVE_ICON_DIR ]; then	# only create rox app when one doesn't exist
				mkdir -p $DRIVE_ICON_DIR
				get_summary $FS_TYPE # returns SUMMARY
				create_rox_app $DRIVE_ICON_DIR $DRIVE_TYPE $DEVNAME "$SUMMARY"
			 fi
			
			 get_mutex
			 if ! grep -F $DRIVE_ICON_DIR $PINBOARD_FILE; then # only add icon when one doesn't exist
				 . $EVENTMANAGER_CONFIG
				 get_free_coord $ICON_ALIGNMENT # returnx COORD_X, COORD_Y
				 rox_add_icon $DRIVE_ICON_DIR $COORD_X $COORD_Y $DRIVENAME "click $DEVNAME"
				 fatdog-drive-icon-refresh-icon.sh $DEVNAME		 				 
			 fi
			 release_mutex
			 ;;
	esac
}

### add handler - remove icons to rox
udev_remove_handler() {
	get_mutex
	rox_remove_icon $DRIVE_ICON_DIR
	release_mutex
	rm -rf $DRIVE_ICON_DIR
}

### change handler - add/remove icons to rox
udev_change_handler() {
	case $DRIVENAME in
		sr*) # special treatment for optical discs
			if cddetect_quick -d $DEVNAME; then
				udev_add_handler		# disc inserted - add icon
			else
				udev_remove_handler		# anything else - remove
			fi	
			;;
		*) # for others - remove then add (refresh it)
		   udev_remove_handler
		   udev_add_handler
		   ;;
	esac
}

#################### main #####################
#DEVPATH=/devices/pci0000:00/0000:00:1f.2/host1/target1:0:0/1:0:0:0/block/sr0
#DEVNAME=/dev/sr0
setup_env 		 # DISPLAY, USER, HOME, DRIVE_ICON_ROOT, PINBOARD_FILE
[ ! -e $F_ENABLE ] && exit 

DRIVENAME=${DEVNAME##*/}
DRIVE_ICON_DIR=$DRIVE_ICON_ROOT/$DRIVENAME
guess_drive_type $DRIVENAME $DEVPATH 	# returns DRIVE_TYPE
case $DRIVENAME in loop*) exit ;; esac	# avoid showing loop device

case $ACTION in
	add)	udev_add_handler ;;
	remove)	udev_remove_handler ;;
	change) udev_change_handler ;;
esac
